#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import datetime as dt
import confuse

from os import path
from peewee import (
    SqliteDatabase,
    Model,
    CharField,
    IntegerField,
    BooleanField,
    DateTimeField,
    FloatField,
)


DB = None


def database():
    config = confuse.Configuration("pawnstore", __name__)

    return SqliteDatabase(path.join(config.config_dir(), "pawnstore.sqlite"))


class Game(Model):
    slug = CharField(unique=True)

    analysis = CharField(null=True)
    eco = CharField()
    eco_name = CharField(null=True)
    elo = IntegerField(null=True)
    moves = CharField()
    num_moves = IntegerField(null=True)
    opp_elo = IntegerField()
    opp_name = CharField()
    pgn = CharField(null=True)
    result = CharField(null=True)
    termination = CharField(null=True)
    speed = CharField()
    term = CharField(null=True)
    timestamp = DateTimeField()
    user = CharField()
    accuracy = FloatField(null=True)
    time_control = CharField()
    website = CharField()
    white = BooleanField()
    searchable_fields = ("eco", "opp_name", "website", "white")

    class Meta:
        database = database()
        indexes = (
            # create a unique on from/to/date
            (("slug", "website", "user"), True),
        )

    def __str__(self):
        if self.white:
            return f"{self.user} x {self.opp_name}"
        return f"{self.opp_name} x {self.user}"


# Initialize database
database().create_tables([Game])



import logging
from .models import Game

from pawnstore.chesscom import CHESSCOM
from pawnstore.lichess import LICHESS

__app__ = "pawnstore"
__version__ = "0.1.0"

logger = logging.getLogger(__name__)


def pawnstore(chesscom=None, lichess=None, full=False):

    if chesscom:
        CHESSCOM.sync(chesscom, full)

    if lichess:
        LICHESS.sync(lichess, full)

    return (
        Game.select()
        .where(
            ((Game.user == chesscom) & (Game.website == CHESSCOM.name))
            | ((Game.user == lichess) & (Game.website == LICHESS.name))
        )
        .order_by(Game.timestamp)
    )



import io
import chess.pgn

from peewee import IntegrityError

from .models import Game


def parse_game(game):
    hdr = {**game.headers._tag_roster, **game.headers._others}
    board = chess.Board()
    moves = []
    for x in game.mainline_moves():
        moves.append(board.san(x))
        board.push(x)
    return hdr, moves


def insert_game(game_dict):
    """Insert game in database"""
    if not game_dict or not game_dict.get("slug"):  # ??
        return False
    game = Game.create(**game_dict)
    return True


class ChessPlatform:
    def export_by_player(self, username, num=None):
        """Yield num last games of username as json blobs"""
        raise NotImplementedError

    def user_centric(self, json, user):
        """ """
        raise NotImplementedError

    def sync(self, user, full):
        inserts = 0

        for game in self.export_by_player(user, 100):
            game_dict = self.user_centric(game, user)
            if game_dict:
                try:
                    inserts += insert_game(game_dict)
                except IntegrityError as e:
                    if not full:
                        break  # reached last insert game
        if inserts:
            print(f"{inserts} {self.name} games inserted")



import io

import chess.pgn
import requests
import datetime as dt

from pawnstore.services import ChessPlatform, parse_game, insert_game

ARCHIVES_URL = "https://api.chess.com/pub/player/{}/games/archives"


def opposite(color):
    return "Black" if color.title() == "White" else "White"


class ChesscomError(Exception):
    pass


class Chesscom(ChessPlatform):
    name = "chess.com"
    convert = {"TimeControl": {"300+5": "5+5", "600": "10+0", "900+10": "15+10"}}

    def export_by_player(self, username, num=None, desc=True):
        """Yield num last games of username as json blobs"""

        idx = 0
        try:
            res = requests.get(ARCHIVES_URL.format(username)).json()
        except requests.exceptions.ConnectionError as e:
            print("Connection error: skipping {} import".format(self.name))
            return
        if res.get("code") == 0:
            raise ChesscomError(res["message"])

        archives = res["archives"]
        order = -1 if desc else 1
        for monthly_url in archives[::order]:
            res = requests.get(monthly_url).json()
            games = res["games"][::-1]

            for game in games:
                yield game
                idx += 1
                if idx == num:
                    return

    def _user_result(self, json, color):
        """Return result of the game for side with given color."""
        if json["white"]["result"] == "win":
            termination = json["black"]["result"]
            return ("W" if color == "white" else "L", termination)
        elif json["black"]["result"] == "win":
            termination = json["white"]["result"]
            return ("W" if color == "black" else "L", termination)
        else:  # draw

            return ("D", json[color]["result"])

    def _user_accuracy(self, json, color):
        accuracies = json.get("accuracies")
        if accuracies:
            return accuracies[color]

    def _opening_name(self, url):
        words = url.split("/")[-1].replace("-", " ").split()
        res = []
        for item in words:

            if item[0].isdigit():
                break
            res.append(item)
        return " ".join(res)

    def user_centric(self, json, user):
        """Return game as a dict with fields adopting a user POV."""

        res = {
            "slug": json.get("url", None),
            "pgn": json.get("pgn", None),
            "website": self.name,
            "speed": "bullet",  # TODO
        }

        hdr, moves = parse_game(chess.pgn.read_game(io.StringIO(json["pgn"])))
        color = "white" if (hdr["White"].lower() == user.lower()) else "black"
        res["white"] = color == "white"
        res["eco"] = hdr["ECO"]
        res["eco_name"] = self._opening_name(hdr["ECOUrl"])
        res["elo"] = hdr[color.title() + "Elo"]

        res["opp_name"] = hdr[opposite(color)]

        res["opp_elo"] = hdr[opposite(color) + "Elo"]
        res["time_control"] = self.convert["TimeControl"].get(
            hdr["TimeControl"], hdr["TimeControl"]
        )
        res["user"] = user
        res["accuracy"] = self._user_accuracy(json, color)
        res["moves"] = " ".join(moves)

        res["timestamp"] = dt.datetime.fromtimestamp(json["end_time"])
        res["result"], res["termination"] = self._user_result(json, color.lower())
        return res


CHESSCOM = Chesscom()



import io
import os

import chess.pgn
import requests
import berserk

import datetime as dt

from pawnstore.services import ChessPlatform, parse_game

LICHESS_API = berserk.Client(berserk.TokenSession(os.environ.get("LICHESS_API_TOKEN")))


class LichessError(Exception):
    pass


class Lichess(ChessPlatform):
    name = "lichess.org"
    convert = {
        "Termination": {
            "mate": "checkmated",
            "resign": "resigned",
            "draw": "",
            "outoftime": "timeout",
        }
    }

    def export_by_player(self, username, max=None):
        return LICHESS_API.games.export_by_player(
            username, as_pgn=False, moves=True, evals=True, opening=True, max=max
        )

    def _user_result(self, json, color):
        """Return result of the game for side with given color."""
        res = "L"
        if json["status"] in ("draw", "stalemate"):
            res = "D"
        elif json["winner"] == color:
            res = "W"
        return res

    def user_centric(self, json, user):
        """Return game as a dict with fields adopting a user POV."""
        game = json
        if "opening" not in json:
            return
        res = {}
        mapping = {
            "createdAt": "timestamp",
            "winner": "result",
        }
        for key in (
            "variant",
            "speed",
            "createdAt",
            "winner",
            "moves",
            "analysis",
            "pgn",
        ):
            res[mapping.get(key, key)] = json.get(key, None)

        res["slug"] = f"https://lichess.org/{json['id']}"
        res["eco"] = json["opening"]["eco"]
        res["eco_name"] = json["opening"]["name"]

        try:
            if json["players"]["white"]["user"]["id"] == user.lower():
                color, opp_color = "white", "black"
            else:
                color, opp_color = "black", "white"
        except KeyError:  # game against AI
            return

        res["white"] = color == "white"
        res["elo"] = json["players"][color]["rating"]
        res["opp_name"] = json["players"][opp_color]["user"]["id"]
        res["opp_elo"] = json["players"][opp_color]["rating"]
        res["website"] = self.name
        res["user"] = user
        res["result"] = self._user_result(json, color)
        res["termination"] = self.convert["Termination"].get(json["status"])
        res["time_control"] = "{total}+{incr}".format(
            total=int(json["clock"]["initial"] / 60), incr=json["clock"]["increment"]
        )

        return res


LICHESS = Lichess()



#!/usr/bin/env python3

# Copyright (c) 2022 Fabrice Laporte - kray.me
# The MIT License http://www.opensource.org/licenses/mit-license.php

import codecs
import os
import re
from setuptools import setup


PKG_NAME = "pawnstore"
DIRPATH = os.path.dirname(__file__)


def read_rsrc(filename, pypi_compat=False):
    """Return content of filename.
    If pypi_compat is True, remove emojis and anything preceding
    `.. pypi` comment if present.
    """
    with codecs.open(os.path.join(DIRPATH, filename), encoding="utf-8") as _file:
        data = _file.read().strip()
        if pypi_compat or filename == "README.rst":
            data = re.sub(r":(\w+\\?)+:", "", data[data.find(".. pypi") :] or data)
    return data


with codecs.open("{}/__init__.py".format(PKG_NAME), encoding="utf-8") as fd:
    version = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fd.read(), re.MULTILINE
    ).group(1)

# Deploy: python3 setup.py sdist bdist_wheel; twine upload --verbose dist/*
setup(
    name=PKG_NAME,
    version=version,
    description="chess library to import your PGN games in a local database",
    long_description=read_rsrc("README.rst"),
    author="Fabrice Laporte",
    author_email="kraymer@gmail.com",
    url="https://github.com/KraYmer/pawnstore",
    license="MIT",
    platforms="ALL",
    packages=[
        "pawnstore",
    ],
    install_requires=read_rsrc("requirements.txt").split("\n"),
    python_requires=">=3.6",
    extras_require={
        "test": [
            "coverage>5",
            "pytest>=6",
            "tox>=3",
        ]
    },
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Environment :: Console",
        "Topic :: Games/Entertainment :: Board Games",
    ],
    keywords="chess",
)
